/*
  Simple MPI program that sends a single message in a ring from Process
  i to Process (i+1)%numprocs

  Jim Teresco
  Department of Computer Science
  Williams College

  Last Modification:
  Wed Feb 26 17:14:46 EST 2003

  $Id$
*/

#include <stdio.h>
#include <mpi.h>

int main(int argc, char *argv[]) {
  int rc;
  MPI_Status status;
  MPI_Request send_request, recv_request;
  int mypid, numprocs;
  int message;
  int to, from;

  /* Initialize MPI */
  rc = MPI_Init(&argc,&argv);
  if (rc != MPI_SUCCESS) {
    fprintf(stderr,"Cannot initialize MPI\n");
    MPI_Abort(MPI_COMM_WORLD, 1);
  }

  /* Query MPI for our rank and the number of procs */
  MPI_Comm_rank(MPI_COMM_WORLD,&mypid);
  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);

  /* we send a message "up" to the next highest rank process, unless
     we're the highest rank, in which case we send to process 0 */
  to = (mypid+1)%numprocs;

  /* we receive our message from the next lower rank process, except
     at process 0 where we receive from the highest rank process */
  from = mypid-1;
  if (from == -1) from = numprocs-1;

  /* send my message - just our pid */
  MPI_Isend(&mypid,1,MPI_INT,to,1,MPI_COMM_WORLD,&send_request);

  /* post the corresponding recv */
  MPI_Irecv(&message,1,MPI_INT,from,1,MPI_COMM_WORLD,&recv_request);

  /* wait for completion -- the above are nonblocking calls! */
  MPI_Wait(&send_request,&status);
  MPI_Wait(&recv_request,&status);

  printf("Proc %d received %d from proc %d\n",mypid,message,from);

  MPI_Finalize();
}

